searchData={"items":[{"type":"module","title":"Bind","doc":"`Bind` provides functionality to build dynamic Ecto queries based on given parameters.\nIt allows developers to retrieve data flexibly without writing custom queries for each use case.","ref":"Bind.html"},{"type":"module","title":"Examples - Bind","doc":"Given an Ecto schema module `MyApp.User` and a map of query parameters, you can build and run a query like this:\n\n    > params = %{ \"name[eq]\" => \"Alice\", \"age[gte]\" => 30, \"sort\" => \"-age\", \"limit\" => \"10\" }\n    > query = Bind.query(params, MyApp.User)\n    > results = Repo.all(query)\n    > IO.inspect(results)\n\nOr, you can pipe:\n\n    > %{ \"name[eq]\" => \"Alice\", \"age[gte]\" => 30, \"sort\" => \"-age\", \"limit\" => \"10\" }\n    |> Bind.query(MyApp.User)\n    |> Repo.all()\n    |> IO.inspect()\n\nIf you're in a Phoenix controller:\n\n    def index(conn, params) do\n      users = conn.query_string\n        |> Bind.decode_query()\n        |> Bind.query(MyApp.User)\n        |> Repo.all()\n\nrender(conn, \"index.json\", users: users)\n    end","ref":"Bind.html#module-examples"},{"type":"function","title":"Bind.filter/2","doc":"Merges additional filters with existing query params.","ref":"Bind.html#filter/2"},{"type":"function","title":"Examples - Bind.filter/2","doc":"# Only allow users to see their own posts\n    conn.query_string\n      |> Bind.filter(%{\"user_id[eq]\" => current_user.id})\n      |> Bind.query(Post)\n\n    # Scope to team and active status\n    params\n      |> Bind.filter(%{\"team_id[eq]\" => team_id})\n      |> Bind.filter(%{\"active[true]\" => true})\n      |> Bind.query(Post)","ref":"Bind.html#filter/2-examples"},{"type":"function","title":"Bind.map/2","doc":"Maps over query parameters using field transformer functions.","ref":"Bind.html#map/2"},{"type":"function","title":"Parameters - Bind.map/2","doc":"- `params`: Map of query parameters (e.g. %{\"user_id[eq]\" => \"123\"})\n - `field_mappers`: Map of field names to transformer functions.\n    Can contain atom keys for exact matches or regex patterns for flexible matching.","ref":"Bind.html#map/2-parameters"},{"type":"function","title":"Examples - Bind.map/2","doc":"# Transform specific fields\n   params = %{\"user_id[eq]\" => \"123\", \"name[eq]\" => \"alice\"}\n   Bind.map(params, %{\n     user_id: fn id -> HashIds.decode(id) end,\n     name: &String.upcase/1\n   })\n   # => %{\"user_id[eq]\" => \"u_123\", \"name[eq]\" => \"ALICE\"}\n\n   # Transform multiple fields with regex pattern\n   params = %{\"user_id[eq]\" => \"123\", \"team_id[eq]\" => \"456\"}\n   Bind.map(params, %{\n     ~r/_id$/i => fn id -> HashIds.decode(id) end\n   })\n   # => %{\"user_id[eq]\" => \"u_123\", \"team_id[eq]\" => \"t_456\"}\n\nNote: Only transforms values for where conditions (e.g. [eq], [gte]).\nOther parameters like sort, limit etc. are preserved unchanged.","ref":"Bind.html#map/2-examples"},{"type":"function","title":"Bind.query/2","doc":"Builds an Ecto query for the given schema based on the provided query string.","ref":"Bind.html#query/2"},{"type":"function","title":"Parameters - Bind.query/2","doc":"- `query_string`: The query string from a URL.\n  - `schema`: The Ecto schema module (e.g., `MyApp.User`).","ref":"Bind.html#query/2-parameters"},{"type":"function","title":"Examples - Bind.query/2","doc":"> query_string = \"?name[eq]=Alice&age[gte]=30&sort=-age&limit=10\"\n    > Bind.query(query_string, MyApp.User)\n    #Ecto.Query = ^30, order_by: [desc: u0.age]>","ref":"Bind.html#query/2-examples"},{"type":"module","title":"Bind.Parse","doc":"","ref":"Bind.Parse.html"},{"type":"function","title":"Bind.Parse.sort_field/1","doc":"Parses the sort parameter to determine the sort direction and field.","ref":"Bind.Parse.html#sort_field/1"},{"type":"function","title":"Parameters - Bind.Parse.sort_field/1","doc":"- `param`: The sort parameter as a string.","ref":"Bind.Parse.html#sort_field/1-parameters"},{"type":"function","title":"Examples - Bind.Parse.sort_field/1","doc":"> Bind.Parse.sort_field(\"-age\")\n    [desc: :age]\n\n    > Bind.Parse.sort_field(\"name\")\n    [asc: :name]","ref":"Bind.Parse.html#sort_field/1-examples"},{"type":"function","title":"Bind.Parse.where_field/1","doc":"Parses a where parameter to extract the field name and constraint.","ref":"Bind.Parse.html#where_field/1"},{"type":"function","title":"Parameters - Bind.Parse.where_field/1","doc":"- `param`: The where parameter as a string.","ref":"Bind.Parse.html#where_field/1-parameters"},{"type":"function","title":"Examples - Bind.Parse.where_field/1","doc":"> Bind.Parse.where_field(\"name[eq]\")\n    [:name, \"eq\"]\n\n    > Bind.Parse.where_field(\"age[gte]\")\n    [:age, \"gte\"]","ref":"Bind.Parse.html#where_field/1-examples"},{"type":"module","title":"Bind.QueryBuilder","doc":"","ref":"Bind.QueryBuilder.html"},{"type":"function","title":"Bind.QueryBuilder.add_limit_query/2","doc":"","ref":"Bind.QueryBuilder.html#add_limit_query/2"},{"type":"function","title":"Bind.QueryBuilder.add_offset_query/2","doc":"","ref":"Bind.QueryBuilder.html#add_offset_query/2"},{"type":"function","title":"Bind.QueryBuilder.build_sort_query/1","doc":"","ref":"Bind.QueryBuilder.html#build_sort_query/1"},{"type":"function","title":"Bind.QueryBuilder.build_where_query/1","doc":"","ref":"Bind.QueryBuilder.html#build_where_query/1"},{"type":"function","title":"Bind.QueryBuilder.constraint/3","doc":"Parses a constraint and returns a dynamic query fragment.","ref":"Bind.QueryBuilder.html#constraint/3"},{"type":"function","title":"Parameters - Bind.QueryBuilder.constraint/3","doc":"- `field`: The field to apply the constraint to.\n  - `constraint`: The type of constraint (e.g., \"eq\", \"gte\").\n  - `value`: The value to compare the field against.","ref":"Bind.QueryBuilder.html#constraint/3-parameters"},{"type":"function","title":"Examples - Bind.QueryBuilder.constraint/3","doc":"> Bind.Parse.constraint(:name, \"eq\", \"Alice\")\n    dynamic([r], r.name == ^\"Alice\")\n\n    > Bind.Parse.constraint(:age, \"gte\", 30)\n    dynamic([r], r.age > ^30)","ref":"Bind.QueryBuilder.html#constraint/3-examples"},{"type":"function","title":"Bind.QueryBuilder.validate_where_query/1","doc":"","ref":"Bind.QueryBuilder.html#validate_where_query/1"},{"type":"module","title":"Bind.QueryString","doc":"Provides functionality to convert a query string from a URL into a map.","ref":"Bind.QueryString.html"},{"type":"function","title":"Bind.QueryString.to_map/1","doc":"Converts a query string from a URL into a map, converting numerical values to number type.","ref":"Bind.QueryString.html#to_map/1"},{"type":"function","title":"Parameters - Bind.QueryString.to_map/1","doc":"- `query_string`: The query string from a URL.","ref":"Bind.QueryString.html#to_map/1-parameters"},{"type":"function","title":"Examples - Bind.QueryString.to_map/1","doc":"iex> Bind.QueryString.to_map(\"?name[eq]=Alice&age[gte]=30&sort=-age&limit=10\")\n    %{\"name[eq]\" => \"Alice\", \"age[gte]\" => 30, \"sort\" => \"-age\", \"limit\" => 10}","ref":"Bind.QueryString.html#to_map/1-examples"},{"type":"extras","title":"Bind","doc":"# Bind\n\nDefine an API controller like this:\n\n```ex\ndef index(conn, _params) do\n  users = conn.query_string\n    |> Bind.query(User)\n    |> Repo.all()\n\n  render(conn, :index, result: users)\nend\n```\n\nNow your endpoint supports all these queries out of the box:\n\n```\nGET /users?name[contains]=john&sort=-id&limit=25\nGET /users?salary[gte]=50000&location[eq]=berlin\nGET /users?joined_at[lt]=2024-01-01&status[neq]=disabled\n```\n\nBind is a flexible and dynamic Ecto query builder, for retrieving data flexibly without writing custom queries for each use case.","ref":"readme.html"},{"type":"extras","title":"Installation - Bind","doc":"Add `bind` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:bind, \"~> 0.1.1\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"API - Bind","doc":"```ex\nBind.query(schema, params)\n```\n\nParameters:\n\n-   `schema`: The Ecto schema module (e.g., `MyApp.User`).\n-   `params`: Either a map of query parameters or a query string.\n\nReturns: An Ecto query.","ref":"readme.html#api"},{"type":"extras","title":"Usage Example - Bind","doc":"Create Ecto query:\n\n```ex\nquery = Bind.query(%{ \"name[eq]\" => \"Alice\", \"age[gte]\" => 30 }, MyApp.User)\n```\n\nAlternatively, with a query string:\n\n```ex\nquery = Bind.query(\"?name[eq]=Alice&age[gte]=30\", MyApp.User)\n```\n\nAnd finally run the query to get results from the database:\n\n```ex\nresults = Repo.all(query)\n```\n\nHere's how it looks in a controller:\n\n```ex\ndef index(conn, params) do\n  images = conn.query_string\n    |> Bind.query(MyApp.Media.Image)\n    |> MyApp.Repo.all()\n\n  render(conn, :index, result: images)\nend\n```\n\nError handling\n\n```ex\ncase Bind.query(%{ \"name[eq]\" => \"Alice\", \"age[gte]\" => 30 }, MyApp.User) do\n  {:error, reason} ->\n    IO.puts(\"Error building query: #{reason}\")\n\n  query ->\n    results = Repo.all(query)\nend\n```","ref":"readme.html#usage-example"},{"type":"extras","title":"Filtering - Bind","doc":"Examples:\n\n```ex\n%{\"name[eq]\" => \"Alice\", \"age[gte]\" => 30}\n```\n\n```ex\n%{\n  \"name[starts_with]\" => \"A\",\n  \"age[gte]\" => 18,\n  \"role[in]\" => \"superuser,admin,mod\",\n  \"is_active[true]\" => \"\",\n  \"last_login[nil]\" => false\n}\n```\n\nList of comparison operators supported:\n\n-   `eq`: Equal to\n-   `neq`: Not equal to\n-   `gt`: Greater than\n-   `gte`: Greater than or equal to\n-   `lt`: Less than\n-   `lte`: Less than or equal to\n-   `true`: Boolean true\n-   `false`: Boolean false\n-   `starts_with`: String starts with\n-   `ends_with`: String ends with\n-   `in`: In a list of values\n-   `contains`: String contains\n-   `nil`: Is nil (or is not nil)","ref":"readme.html#filtering"},{"type":"extras","title":"Sorting - Bind","doc":"Use the `sort` parameter to specify sorting order:\n\n-   Prefix with `-` for descending order\n-   No prefix for ascending order\n\n```ex\n%{\"sort\" => \"-age\"}  # Sort by age descending\n%{\"sort\" => \"age\"}  # Sort by age ascending\n```\n\nIf nothing specified, sorts by ID field ascending.","ref":"readme.html#sorting"},{"type":"extras","title":"Pagination - Bind","doc":"-   `limit`: Specify the maximum number of results (default: 10)\n-   `start`: Specify the starting ID for pagination\n\nExample:\n\n```ex\n%{\"limit\" => 20, \"start\" => 100}\n```\n\n- For ascending order (default), use positive ID: `start=100`\n- For descending order, use negative ID: `start=-100`\n\nExample for descending pagination:\n```\nGET /users?sort=-created_at&start=-100\n```","ref":"readme.html#pagination"},{"type":"extras","title":"Query String Support - Bind","doc":"In a typical Phoenix controller, you can simply pass `conn.query_string` and get Ecto query back:\n\n```ex\nquery_string = conn.query_string\n    |> Bind.query(query_string, MyApp.User)\n    |> MyApp.Repo.all()\n```","ref":"readme.html#query-string-support"},{"type":"extras","title":"Transforming Query Parameters - Bind","doc":"You can transform filter values before query is built:\n\n```ex\n\"user_id[eq]=123&team_id[eq]=456\"\n  |> Bind.map(%{\n    user_id: fn id -> HashIds.decode(id) end,\n    team_id: fn id -> HashIds.decode(id) end\n  })\n  |> Bind.query(MyApp.User)\n  |> Repo.all()\n```\n\n\nTransform specific fields\n\n```ex\nBind.map(params, %{\n  user_id: fn id -> HashIds.decode(id) end,\n  name: &String.upcase/1\n})\n```\n\nTransform multiple fields with regex pattern:\n\n```ex\nBind.map(params, %{\n  ~r/_id$/i => fn id -> HashIds.decode(id) end\n})\n```\n\nNote: Value transformation only applies to filter fields (e.g. [eq], [gte]), not to sort/limit/pagination params.","ref":"readme.html#transforming-query-parameters"},{"type":"extras","title":"Access Control with Filters - Bind","doc":"You can use filters to enforce access control and limit what users can query. Filters compose nicely with the query builder:\n\n```ex\ndef index(conn, _params) do\n  my_posts = conn.query_string\n    # User can only see their own posts\n    |> Bind.filter(%{\"user_id[eq]\" => conn.assigns.current_user.id})\n    # That are active\n    |> Bind.filter(%{\"active[true]\" => true})\n    |> Bind.query(Post)\n    |> Repo.all()\n\n  render(conn, :index, posts: my_posts)\nend\n```","ref":"readme.html#access-control-with-filters"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}